
#include <gtkmm.h>


class MyButton : public Gtk::Button
{
  public:
    MyButton(const Glib::ustring str);
    ~MyButton(){};
    
    virtual void on_drag_begin (const Glib::RefPtr<Gdk::DragContext>& context);
    virtual void on_drag_data_delete (const Glib::RefPtr<Gdk::DragContext>& context);
    virtual void on_drag_data_get (const Glib::RefPtr<Gdk::DragContext>& context, Gtk::SelectionData& selection_data, guint info, guint time);
    virtual void on_drag_data_received (const Glib::RefPtr<Gdk::DragContext>& context, int x, int y, const Gtk::SelectionData& selection_data, guint info, guint time);
    virtual bool on_drag_drop (const Glib::RefPtr<Gdk::DragContext>& context, int x, int y, guint time);
    virtual void on_drag_end (const Glib::RefPtr<Gdk::DragContext>& context);
    virtual void on_drag_leave (const Glib::RefPtr<Gdk::DragContext>& context, guint time);
    virtual bool on_drag_motion (const Glib::RefPtr<Gdk::DragContext>& context, int x, int y, guint time);
};


int main(int argc, char *argv[])
{
  Gtk::Main kit(argc, argv);

  Gtk::Window win;
  win.set_title("DragNDrop");
  win.set_border_width(10);

  MyButton m_button1("1"), m_button2("2"), m_button3("3");
  Gtk::VBox m_box;
  
  m_button1.show();
  m_button2.show();
  m_button3.show();
  
  bool expand=true;
  bool fill=true;
  int padding=10;
  
  m_box.pack_start(m_button1, expand, fill, padding);
  m_box.pack_start(m_button2, expand, fill, padding);
  m_box.pack_start(m_button3, expand, fill, padding);
  
  m_box.show_all();
  
  win.add(m_box);

  Gtk::Main::run(win);

  return 0;
};


MyButton::MyButton(const Glib::ustring str) : Gtk::Button(str)
{
  //Targets:
  std::list<Gtk::TargetEntry> listTargets;
  listTargets.push_back( Gtk::TargetEntry("text/plain") );

  //Make a DnD drag source:
  drag_source_set(listTargets);

  //Make a DnD drop destination:
  drag_dest_set(listTargets);
}


void MyButton::on_drag_begin (const Glib::RefPtr<Gdk::DragContext>& context)
{
  printf( "drag begin [%x]\n", (unsigned int)this );
}

void MyButton::on_drag_data_delete (const Glib::RefPtr<Gdk::DragContext>& context)
{
  printf( "drag delete [%x]\n", (unsigned int)this );
}

void MyButton::on_drag_data_get (const Glib::RefPtr<Gdk::DragContext>& context, Gtk::SelectionData& selection_data, guint info, guint time)
{
  printf( "drag data get [%x]\n", (unsigned int)this );
  selection_data.set_text(get_label());
}

void MyButton::on_drag_data_received (const Glib::RefPtr<Gdk::DragContext>& context, int x, int y, const Gtk::SelectionData& selection_data, guint info, guint time)
{
  printf( "drag data received [%x]\n", (unsigned int)this );
  set_label( selection_data.get_text() );
  context->drag_finish(true, true, time);
}

bool MyButton::on_drag_drop (const Glib::RefPtr<Gdk::DragContext>& context, int x, int y, guint time)
{
  printf( "drag drop [%x]\n", (unsigned int)this );
  context->drop_finish(TRUE, time);
  return true;
}

void MyButton::on_drag_end (const Glib::RefPtr<Gdk::DragContext>& context)
{
  printf( "drag end [%x]\n", (unsigned int)this );
}

void MyButton::on_drag_leave (const Glib::RefPtr<Gdk::DragContext>& context, guint time)
{
  printf( "drag leave [%x]\n", (unsigned int)this );
}

bool MyButton::on_drag_motion (const Glib::RefPtr<Gdk::DragContext>& context, int x, int y, guint time)
{
  printf( "drag motion [%x]\n", (unsigned int)this );
  return true;
}

